<?php 
/**
 * function to recalculate financials for the projects associated to a set of objects
 */
function recalculate_project_financials_of_objects($objects) {
	$project_ot_id = ObjectTypes::findByName('project')->getId();

	// get the projects we need to recalculate financials
	$project_ids = array();
	foreach ($objects as &$object) {
		if ($object instanceof ProjectTask) {
			$project_member = $object->getMemberOfType($project_ot_id);
			if ($project_member instanceof Member) {
				$project_ids[] = $project_member->getId();
			}
		}
	}
	$project_ids = array_filter(array_unique($project_ids));

	// recalculate financials for the projects involved in the task template instantiation
	foreach ($project_ids as $project_member_id) {
		// calculate project's labor financials
		update_estimated_labor_info_for_member_using_member_id($project_member_id);
		update_executed_labor_info_for_member_using_member_id($project_member_id);

		if (Plugins::instance()->isActivePlugin('expenses2')) {
			// calculate project's expense financials
			Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
			update_estimated_expenses_info_for_member_using_member_id($project_member_id);
			update_executed_expenses_info_for_member_by_using_member_id($project_member_id);
		}
	}
}


function recalculate_project_financials_of_objects_in_background($objects) {
	
	$user = logged_user();
	
	if (substr(php_uname(), 0, 7) == "Windows" || !is_exec_available()) {

		recalculate_project_financials_of_objects($objects);
		
	} else {

		$ids = array();
		foreach ($objects as $object) {
			if ($object instanceof ContentDataObject) {
				$ids[] = $object->getId();
			}
		}
		$ids_csv = implode(',', $ids);

		if (count($ids) > 0) {

			$script_path = ROOT . "/plugins/advanced_billing/application/helpers/recalculate_project_financials_of_objects_background.php";
			$command = "".PHP_PATH." $script_path ".ROOT." ".$user->getId()." ".$user->getTwistedToken()." ".$ids_csv;
			exec("$command > /dev/null &");
		}
		
	}
}


/*
*
* ESTIMATED LABOR INFO
*
*/
function update_estimated_labor_info_for_member($object){
    // Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

	// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
	// so the first times we get into this function are not relevant because object still has not its complete data
	// in that case we set this variable to skip these calculations and improve performance
	if (isset($object->dont_calculate_project_financials) && $object->dont_calculate_project_financials) return;

    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_estimated_labor_info_for_member_using_member_id($project_member_id);  
    }
}

function update_estimated_labor_info_for_member_using_member_id($project_member_id, $params = array()){
    // Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

    $calculate_parents = array_var($params, 'calculate_parents', true);
    $labor_info = get_estimated_labor_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $labor_info);
    update_estimated_overall_totals_for_member($project_member_id);
    $project_member = Members::instance()->findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0 && $calculate_parents){
            update_estimated_labor_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_estimated_labor_info_for_member($project_member_id) {

    $labor_info = array(
        'estimated_labor_minutes' => 0, 
        'estimated_labor_billable_price' => 0,
        'estimated_labor_non_billable_price' => 0,
        'estimated_labor_total_price' => 0,
        'estimated_labor_billable_cost' => 0,
        'estimated_labor_non_billable_cost' => 0,
        'estimated_labor_total_cost' => 0
    );
    
    // Get tasks and financials info
	$task_ids = "GROUP_CONCAT(task_id) AS task_ids";
    $estimated_hrs = "SUM(total_time_estimate) AS total_time_estimate ";
    $estimated_cost = "SUM(estimated_cost) AS total_estimated_cost ";
    $estimated_price = "SUM(estimated_price) AS total_estimated_price ";

    $billable_tasks_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("o.id as task_id", "is_billable", "total_time_estimate", "estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $task_ids, $estimated_hrs, $estimated_cost, $estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 AND `is_billable`=1 "
    ))->objects; 
    
    $non_billable_tasks_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("o.id as task_id", "is_billable", "total_time_estimate", "estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $task_ids, $estimated_hrs, $estimated_cost, $estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 AND `is_billable`=0 "
    ))->objects;  

	$billable_task_totals = $billable_tasks_info[0];
	$non_billable_task_totals = $non_billable_tasks_info[0];

	// As tasks financials contains the related expenses amounts, we have to subtract them from its financials to get the labor amounts
	if (Plugins::instance()->isActivePlugin('expenses2')) {
        // Get tasks and subtasks with manual estimated price and estimated cost
        $manual_estimated_price_task_ids = get_tasks_and_subtask_ids_with_manual_financial_value($project_member_id);

		$parent_billable_task_ids = explode(',', $billable_task_totals['task_ids']);
        $all_billable_task_ids = add_subtask_ids_to_array_of_parents($parent_billable_task_ids);

        // Get the cost totals of the expenses related to the involved tasks
        $billable_expense_task_totals_cost = Expenses::getBudgetedExpensesTotalsByTasks($all_billable_task_ids);

        // Get the price totals of the expenses related to the involved tasks
        $filtered_price_billable_task_ids = array_diff($all_billable_task_ids, $manual_estimated_price_task_ids);
		$billable_expense_task_totals_price = Expenses::getBudgetedExpensesTotalsByTasks($filtered_price_billable_task_ids); 

		// subtract the expense totals to get the labor totals
		$billable_task_totals['total_estimated_cost'] -= $billable_expense_task_totals_cost['total_cost'];
		$billable_task_totals['total_estimated_price'] -= $billable_expense_task_totals_price['total_price'];
       
    
		// get the totals of the expenses related to the involved tasks
		$parent_non_billable_task_ids = explode(',', $non_billable_task_totals['task_ids']);
        $all_non_billable_task_ids = add_subtask_ids_to_array_of_parents($parent_non_billable_task_ids);

        // Get the price totals of the expenses related to the non-billable tasks
        $filtered_non_billable_task_ids_price = array_diff($all_non_billable_task_ids, $manual_estimated_price_task_ids);
		$non_billable_expense_task_totals_price = Expenses::getBudgetedExpensesTotalsByTasks($filtered_non_billable_task_ids_price);

        // Get the cost totals of the expenses related to the non-billable tasks
		$non_billable_expense_task_totals_cost = Expenses::getBudgetedExpensesTotalsByTasks($all_non_billable_task_ids);

		// subtract the expense totals to get the labor totals
		$non_billable_task_totals['total_estimated_cost'] -= $non_billable_expense_task_totals_cost['total_cost'];
		$non_billable_task_totals['total_estimated_price'] -= $non_billable_expense_task_totals_price['total_price'];
	}

    calculate_labor_values($labor_info, $billable_task_totals, $non_billable_task_totals);
    
    return $labor_info;
}

function get_object_project_member_id($object) {   
    $project_ot_id = ObjectTypes::findByName('project')->getId();
    $project_members = get_object_members_by_type($object, $project_ot_id);
    $project_member = array_var($project_members, 0);
    $project_member_id = 0;
    if($project_member instanceof Member){
        $project_member_id = $project_member->getId();
    }

    return $project_member_id;
}

function calculate_labor_values(&$labor_info, $billable_tasks_info, $non_billable_tasks_info) {
    // Calculate labor hours
    $labor_info['estimated_labor_minutes'] += $billable_tasks_info['total_time_estimate'] ? $billable_tasks_info['total_time_estimate'] : 0;
    $labor_info['estimated_labor_minutes'] += $non_billable_tasks_info['total_time_estimate'] ? $non_billable_tasks_info['total_time_estimate'] : 0;

    // Calculate estimated labor billable price
    $labor_info['estimated_labor_billable_price'] += $billable_tasks_info['total_estimated_price'] ? $billable_tasks_info['total_estimated_price'] : 0;
    // Calculate estimated labor non-billable price
    $labor_info['estimated_labor_non_billable_price'] += $non_billable_tasks_info['total_estimated_price'] ? $non_billable_tasks_info['total_estimated_price'] : 0;
    // Calculate estimated total price
    $labor_info['estimated_labor_total_price'] += $billable_tasks_info['total_estimated_price'] ? $billable_tasks_info['total_estimated_price'] : 0;
    $labor_info['estimated_labor_total_price'] += $non_billable_tasks_info['total_estimated_price'] ? $non_billable_tasks_info['total_estimated_price'] : 0;

     // Calculate estimated labor billable cost
     $labor_info['estimated_labor_billable_cost'] += $billable_tasks_info['total_estimated_cost'] ? $billable_tasks_info['total_estimated_cost'] : 0;
     // Calculate estimated labor non-billable cost
     $labor_info['estimated_labor_non_billable_cost'] += $non_billable_tasks_info['total_estimated_cost'] ? $non_billable_tasks_info['total_estimated_cost'] : 0;
     // Calculate estimated total cost
     $labor_info['estimated_labor_total_cost'] += $billable_tasks_info['total_estimated_cost'] ? $billable_tasks_info['total_estimated_cost'] : 0;
     $labor_info['estimated_labor_total_cost'] += $non_billable_tasks_info['total_estimated_cost'] ? $non_billable_tasks_info['total_estimated_cost'] : 0;
}

function set_new_member_totals_values($project_member_id, $labor_info) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    foreach($labor_info as $key => $value){
        $member_totals->setColumnValue($key, $value);
    }
    $member_totals->save();
    
}

/*
*
* EXECUTED LABOR INFO
*
*/
function update_executed_labor_info_for_member($object){
    // Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

	// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
	// so the first times we get into this function are not relevant because object still has not its complete data
	// in that case we set this variable to skip these calculations and improve performance
	if (isset($object->dont_calculate_project_financials) && $object->dont_calculate_project_financials) return;

    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_executed_labor_info_for_member_using_member_id($project_member_id);
    }
}

function update_executed_labor_info_for_member_using_member_id($project_member_id, $params = array()){
    // Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

    $calculate_parents = array_var($params, 'calculate_parents', true);
    $exec_labor_info = get_executed_labor_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $exec_labor_info);
    update_executed_overall_totals_for_member($project_member_id);
    $project_member = Members::instance()->findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0 && $calculate_parents){
            update_executed_labor_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_executed_labor_info_for_member($project_member_id) {

    $labor_info = array(
        'executed_labor_minutes' => 0, 
        'executed_labor_billable_minutes' => 0,
        'executed_labor_non_billable_minutes' => 0,
        'executed_labor_billable_price' => 0,
        'executed_labor_non_billable_price' => 0,
        'executed_labor_total_price' => 0,
        'executed_labor_billable_cost' => 0,
        'executed_labor_non_billable_cost' => 0,
        'executed_labor_total_cost' => 0
    );
    
    // Get tasks and financials info
    $executed_hrs = "SUM(worked_time) AS total_worked_time ";
    $executed_cost = "SUM(fixed_cost) AS total_executed_cost ";
    //$executed_price = "SUM(fixed_billing) AS total_executed_price ";

    if (Plugins::instance()->isActivePlugin('income')) {


        $executed_billable_price = calculate_executed_labor_price($project_member_id, true);
        $billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("invoicing_status", "worked_time", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "check_sharing_table" => false,
            "count_results" => false,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `invoicing_status` != 'non_billable' "
        ))->objects; 
        $billable_time_info[0]['total_executed_price'] = $executed_billable_price;


        $non_billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("invoicing_status", "worked_time", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "check_sharing_table" => false,
            "count_results" => false,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `invoicing_status` = 'non_billable' "
        ))->objects; 
        $non_billable_time_info[0]['total_executed_price'] = 0;

        calculate_executed_labor_values($labor_info, $billable_time_info[0],  $non_billable_time_info[0]);

    } else {
        $executed_billable_price = calculate_executed_labor_price($project_member_id, true);
        $billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("worked_time", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "check_sharing_table" => false,
            "count_results" => false,
            "member_ids" => array($project_member_id)
        ))->objects; 
        $billable_time_info[0]['total_executed_price'] = $executed_billable_price;

        $non_billable_time_info = array(
            'total_worked_time' => 0,
            'total_executed_cost' => 0,
            'total_executed_price' => 0
        );

        calculate_executed_labor_values($labor_info, $billable_time_info[0], $non_billable_time_info);
    }
    
    return $labor_info;
}

function calculate_executed_labor_price($project_member_id, $billable = true) {
     // Get task's earned value with manual estimated price
	$task_ids = "GROUP_CONCAT(task_id) AS task_ids";
    $executed_price = "SUM(earned_value) AS total_executed_price";

    $tasks_executed_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("earned_value", "is_calculated_estimated_price"),
        "query_wraper_start" => "SELECT $executed_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 "
    ))->objects; 


    // Get all task ids and it's subtasks with manual estimated price
    $executed_values = true;
    $all_task_ids = get_tasks_and_subtask_ids_with_manual_financial_value($project_member_id, 'is_calculated_estimated_price', $executed_values);

    $time_info_rel_condition = " AND `rel_object_id`=0 ";
    $executed_price = "SUM(fixed_billing) AS total_executed_price ";
    $time_info = Timeslots::instance()->listing(array(
        "select_columns" => array("fixed_billing"),
        "query_wraper_start" => "SELECT $executed_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => $time_info_rel_condition
    ))->objects; 

    $expenses_executed_price = 0;
    if(Plugins::instance()->isActivePlugin('expenses2')) {
        Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
        $extra_conditions = !empty( $all_task_ids) ? ' AND `task_id`!=0 AND `task_id` NOT IN ('.implode(',',  $all_task_ids).')' : ' AND `task_id`!=0';
        $is_billable = $billable ? 1 : 0;
        $billable_price = get_executed_expenses_totals_for_member($project_member_id, 'price', ' AND `is_billable`='.$is_billable . $extra_conditions);
        $expenses_executed_price = $billable_price[0]['total_executed_price'];
    }

    $executed_price = array_var($tasks_executed_info[0], 'total_executed_price', 0) + array_var($time_info[0],'total_executed_price', 0) - $expenses_executed_price;

    return $executed_price;
}

function calculate_executed_labor_values(&$labor_info, $billable_time, $non_billable_time) {
     // Calculate labor hours
     $labor_info['executed_labor_minutes'] += $billable_time['total_worked_time'] ? $billable_time['total_worked_time'] : 0;
     $labor_info['executed_labor_minutes'] += $non_billable_time['total_worked_time'] ? $non_billable_time['total_worked_time'] : 0;
     // Calculate labor billable minutes    
    $labor_info['executed_labor_billable_minutes'] += $billable_time['total_worked_time'] ? $billable_time['total_worked_time'] : 0;
    // Calculate labor non-billable minutes
    $labor_info['executed_labor_non_billable_minutes'] += $non_billable_time['total_worked_time'] ? $non_billable_time['total_worked_time'] : 0;

 
     // Calculate labor billable price
     $labor_info['executed_labor_billable_price'] += $billable_time['total_executed_price'] ? $billable_time['total_executed_price'] : 0;
     // Calculate labor non-billable price
     $labor_info['executed_labor_non_billable_price'] += $non_billable_time['total_executed_price'] ? $non_billable_time['total_executed_price'] : 0;
     // Calculate total price
     $labor_info['executed_labor_total_price'] += $billable_time['total_executed_price'] ? $billable_time['total_executed_price'] : 0;
     $labor_info['executed_labor_total_price'] += $non_billable_time['total_executed_price'] ? $non_billable_time['total_executed_price'] : 0;
 
      // Calculate labor billable cost
      $labor_info['executed_labor_billable_cost'] += $billable_time['total_executed_cost'] ? $billable_time['total_executed_cost'] : 0;
      // Calculate labor non-billable cost
      $labor_info['executed_labor_non_billable_cost'] += $non_billable_time['total_executed_cost'] ? $non_billable_time['total_executed_cost'] : 0;
      // Calculate total cost
      $labor_info['executed_labor_total_cost'] += $billable_time['total_executed_cost'] ? $billable_time['total_executed_cost'] : 0;
      $labor_info['executed_labor_total_cost'] += $non_billable_time['total_executed_cost'] ? $non_billable_time['total_executed_cost'] : 0;
} 

/**
 * Update estimated and executed overall values
 */


function update_estimated_overall_totals_for_member($project_member_id) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    $estimated_overall_billable_price =  $member_totals->getColumnValue('estimated_labor_billable_price') +  $member_totals->getColumnValue('estimated_expense_billable_price');
    $estimated_overall_non_billable_price = $member_totals->getColumnValue('estimated_labor_non_billable_price') +  $member_totals->getColumnValue('estimated_expense_non_billable_price');
    $estimated_overall_total_price = $member_totals->getColumnValue('estimated_labor_total_price') +  $member_totals->getColumnValue('estimated_expense_total_price');
    $estimated_overall_billable_cost = $member_totals->getColumnValue('estimated_labor_billable_cost') +  $member_totals->getColumnValue('estimated_expense_billable_cost');
    $estimated_overall_non_billable_cost = $member_totals->getColumnValue('estimated_labor_non_billable_cost') +  $member_totals->getColumnValue('estimated_expense_non_billable_cost');
    $estimated_overall_total_cost = $member_totals->getColumnValue('estimated_labor_total_cost') +  $member_totals->getColumnValue('estimated_expense_total_cost');

    $member_totals->setColumnValue('estimated_overall_billable_price', $estimated_overall_billable_price);
    $member_totals->setColumnValue('estimated_overall_non_billable_price', $estimated_overall_non_billable_price);
    $member_totals->setColumnValue('estimated_overall_total_price', $estimated_overall_total_price);
    $member_totals->setColumnValue('estimated_overall_billable_cost', $estimated_overall_billable_cost);
    $member_totals->setColumnValue('estimated_overall_non_billable_cost', $estimated_overall_non_billable_cost);
    $member_totals->setColumnValue('estimated_overall_total_cost', $estimated_overall_total_cost);

    $member_totals->save();
    
}

function update_executed_overall_totals_for_member($project_member_id) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    $executed_overall_billable_price =  $member_totals->getColumnValue('executed_labor_billable_price') +  $member_totals->getColumnValue('executed_expense_billable_price');
    $executed_overall_non_billable_price = $member_totals->getColumnValue('executed_labor_non_billable_price') +  $member_totals->getColumnValue('executed_expense_non_billable_price');
    $executed_overall_total_price = $member_totals->getColumnValue('executed_labor_total_price') +  $member_totals->getColumnValue('executed_expense_total_price');
    $executed_overall_billable_cost = $member_totals->getColumnValue('executed_labor_billable_cost') +  $member_totals->getColumnValue('executed_expense_billable_cost');
    $executed_overall_non_billable_cost = $member_totals->getColumnValue('executed_labor_non_billable_cost') +  $member_totals->getColumnValue('executed_expense_non_billable_cost');
    $executed_overall_total_cost = $member_totals->getColumnValue('executed_labor_total_cost') +  $member_totals->getColumnValue('executed_expense_total_cost');

    $member_totals->setColumnValue('executed_overall_billable_price', $executed_overall_billable_price);
    $member_totals->setColumnValue('executed_overall_non_billable_price', $executed_overall_non_billable_price);
    $member_totals->setColumnValue('executed_overall_total_price', $executed_overall_total_price);
    $member_totals->setColumnValue('executed_overall_billable_cost', $executed_overall_billable_cost);
    $member_totals->setColumnValue('executed_overall_non_billable_cost', $executed_overall_non_billable_cost);
    $member_totals->setColumnValue('executed_overall_total_cost', $executed_overall_total_cost);

    $member_totals->save();
    
}

function get_all_member_totals_columns() {

    if(isset($_SESSION['member_financial_columns'])) {
        return $_SESSION['member_financial_columns'];
    } else {
        // Increase group_concat_max_len to get all columns
        DB::execute("SET SESSION group_concat_max_len = 99999;");

        // Get columns names from member totals table
        $sql = "SELECT group_concat(COLUMN_NAME)
                FROM INFORMATION_SCHEMA.COLUMNS
                WHERE TABLE_NAME = '". TABLE_PREFIX . "member_totals';";
        $result = DB::execute($sql);
        $row = $result->fetchRow();

        // Remove id and member_id columns
        $financial_columns = array_unique(array_diff(explode(',', $row['group_concat(COLUMN_NAME)']), array('id', 'member_id')));
        $_SESSION['member_financial_columns'] = $financial_columns;

        return $financial_columns;
    }
    
}

function get_all_member_totals_minutes_columns() {
    return array('estimated_labor_minutes', 'executed_labor_minutes', 'executed_labor_non_billable_minutes', 'executed_labor_billable_minutes');
}


/**
 * PERSIST 'percent_completed' COLUMN
 * 
 */

function update_percent_completed_for_member($task) {
    if($task->getParentId() > 0) return;
    $project_member_id = get_object_project_member_id($task);
    if($project_member_id > 0) {
        update_percent_completed_for_member_using_member_id($project_member_id);
    }
}

function update_percent_completed_for_member_using_member_id($project_member_id) {
    $project_member = Members::instance()->findById($project_member_id);
    if(!$project_member instanceof Member) return;

    // Calculate and set percent completed
    $percent_completed_info = calculate_percent_completed_for_member($project_member);
    set_new_member_totals_values($project_member_id, $percent_completed_info);

    // Update parent member
    $parent_id = $project_member->getParentMemberId();
    if ($parent_id > 0) update_percent_completed_for_member_using_member_id($parent_id);
}

function calculate_percent_completed_for_member($project_member) {
    // Calculate percent completed using estimated time and percent completed of parent tasks
    $project_member_id = $project_member->getId();
    $parent_tasks = ProjectTasks::instance()->listing(array(
        "select_columns" => array("percent_completed", "total_time_estimate"),
        "query_wraper_start" => "SELECT SUM(total_time_estimate * percent_completed) AS total_time_estimate_times_percent_completed, SUM(total_time_estimate) AS tasks_total_time_estimate FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 "
    ))->objects;

    $percent_completed = 0;
    if($parent_tasks[0]['tasks_total_time_estimate'] > 0) {
        $percent_completed = $parent_tasks[0]['total_time_estimate_times_percent_completed'] / $parent_tasks[0]['tasks_total_time_estimate'];
    }

    $result['percent_completed'] = $percent_completed;
    
    return $result;
}

function get_tasks_and_subtask_ids_with_manual_financial_value($project_member_id, $financial_column = 'is_calculated_estimated_price', $executed_values = false) {
    $task_ids_select = "GROUP_CONCAT(task_id) AS task_ids";
    $additional_condition = '';
    if($financial_column == 'is_calculated_estimated_price' && $executed_values) {
        $additional_condition = ' AND `is_fixed_fee`=1 ';
    }

    $tasks_manual_executed_all_hierarchies_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("o.id as task_id", "$financial_column", 'is_fixed_fee'),
        "query_wraper_start" => "SELECT $task_ids_select FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `$financial_column`=0 " . $additional_condition 
    ))->objects; 

    // Get all task ids and it's subtasks with manual estimated price
    $task_ids = explode(',', $tasks_manual_executed_all_hierarchies_info[0]['task_ids']);

    $all_task_ids = add_subtask_ids_to_array_of_parents($task_ids);

    return $all_task_ids;
}

function add_subtask_ids_to_array_of_parents($parent_task_ids) {
    $all_task_ids = array();
    foreach ($parent_task_ids as $task_id) {
        $task = ProjectTasks::instance()->findById($task_id);
        if(!$task instanceof ProjectTask) continue;
        $all_task_ids[] = $task_id;
        $all_subtask_ids = $task->getAllSubtaskIdsInHierarchy();
        $all_task_ids = array_merge($all_task_ids, $all_subtask_ids);
    }
    $all_task_ids = array_unique($all_task_ids);

    return $all_task_ids;
}
