<?php

/**
 * ExcelExportOptions controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExcelExportController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	private $can_use_phpspreadsheet = false;
	
	/**
	 * Construct the ExcelExportOptions
	 *
	 * @access public
	 * @param void
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('reporting');
		
		$this->can_use_phpspreadsheet = self::can_use_php_spreadsheet_lib();
		
		if ($this->can_use_phpspreadsheet) {
			Env::useLibrary('PhpSpreadsheet', 'advanced_core');
		} else {
			Env::useLibrary('PHPExcel', 'advanced_core');
		}
		
	} // __construct
	
	
	/**
	 * Returns true if all requirements are met to use PhpSpreadsheet library
	 * @return boolean
	 */
	static function can_use_php_spreadsheet_lib() {
		
		$requirements = array(
				'PHP 7.1.0' => version_compare(PHP_VERSION, '7.1.0', '>='),
				'PHP extension XML' => extension_loaded('xml'),
				'PHP extension xmlwriter' => extension_loaded('xmlwriter'),
				'PHP extension mbstring' => extension_loaded('mbstring'),
				'PHP extension ZipArchive' => extension_loaded('zip'),
				'PHP extension GD (optional)' => extension_loaded('gd'),
				'PHP extension dom (optional)' => extension_loaded('dom'),
		);
		
		$ok = true;
		
		foreach ($requirements as $req) {
			if (!$req) $ok = false;
		}
		
		return $ok;
	}

	private function get_custom_report_filename_to_download($report) {
		if ($report instanceof Report) {
			return str_replace('/', '-', $report->getObjectName()) . ".xlsx";
		}
		return 'report.xlsx';
	}
	
	
	function export_custom_report_excel() {
		ini_set('memory_limit', '1G');
		
		ajx_current("empty");
		$report_id = get_id();
		
		$report = Reports::instance()->findById($report_id);
		$filename = $this->get_custom_report_filename_to_download($report);
		/**
		 * Check if the file was found.
		 */
		if(!$filename)
		{
			ajx_extra_data(array("filename" => "no data"));
			return;
		}

		$file_path = $this->generate_excel_file_from_report($report);
		$size = filesize($file_path);
		
		ajx_extra_data(array("filename" => $filename));
	}
	
	
	function generate_excel_file_from_report(Report $report) {
		// report params
		$params_str = array_var($_REQUEST, 'report_params');
		/**
		 * Create the params variable. If the variable is created inside of the 
		 * if conditional, most of PHP servers declare it as a local variable.
		 */
		$params = null;
		if ($params_str) $params = json_decode(str_replace("'", '"',$params_str), true);
		$_GET['limit'] = 0;
		
		// execute report
		$rc = new ReportingController();
		$_REQUEST['limit'] = 0;
		$results = $rc->view_custom_report($report->getId(), null, $params);

		// get report result in arrays
		$all_data = report_values_to_arrays($results, $report);
		$headers = array_var($all_data, 'headers');
		$all_data_rows = array_var($all_data, 'values');

		// build a sheet for the excel file with the report data 
		$excel_report_sheet_options = array('title' => $report->getObjectName());
		Hook::fire('override_excel_custom_report_sheet_options', array('report' => $report), $excel_report_sheet_options);
		$sheets = array(
			array('headers' => $headers, 'data' => $all_data_rows, 'options' => $excel_report_sheet_options),
		);
		
		$excel_report_options = array('filename' => $this->get_custom_report_filename_to_download($report));
		Hook::fire('override_excel_custom_report_options', array('report' => $report), $excel_report_options);
		
		// generate the excel file
		$filepath = $this->generate_excel_file($sheets, $excel_report_options, $report);
		
		return $filepath;
	}

	// Auxiliar functions for excel export -------------------------------------
	private function excelColumnName($number) {
		$ordA = ord('A');
		$ordZ = ord('Z');
		$len = $ordZ - $ordA + 1;

		$result = '';
		while ($number >= 0) {
			$result = chr($number % $len + $ordA) . $result;
			$number = intval($number / $len) - 1;
		}
		return $result;
	}

	public function addSpaces($count)
	{
		$string = '';

		for ($i = 0; $i < $count; $i++) {
			if($count <= 1):
				$string .= '';
			else:
				$string .= '&nbsp;&nbsp;&nbsp;';
			endif;
		}

		return html_entity_decode($string);
	}

	// private function removeCurrencies($element) {

	// 	$currencies = Currencies::instance()->findAll();

	// 	foreach ($currencies as $currency) :
	// 		$verification = preg_match("'" . $currency->getSymbol() . "'", $element);
	// 		if ($verification) :
	// 			return $element = trim(str_replace($currency->getSymbol(), '', $element));
	// 		else:
	// 			return $element;
	// 		endif;
	// 	endforeach;
	// }
	
	private function get_spreadsheet_writer($spreadsheet) {
		
		if ($this->can_use_phpspreadsheet) {
			$writer = PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, 'Xlsx');
			
		} else {
			// use this zip class because php7 may not include the ZipArchive class by default
			PHPExcel_Settings::setZipClass(PHPExcel_Settings::PCLZIP);
			$writer = PHPExcel_IOFactory::createWriter($spreadsheet, 'Excel2007');
		}
		
		return $writer;
	}
	
	private function create_spreadsheet_object() {
		
		if ($this->can_use_phpspreadsheet) {
			$e = new PhpOffice\PhpSpreadsheet\Spreadsheet();
		} else {
			$e = new PHPExcel();
		}
		
		return $e;
	}
	
	private function get_last_column_letter_by_index($total_cols) {
		
		if ($this->can_use_phpspreadsheet) {
			$last_column_letter = PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($total_cols);
		} else {
			$last_column_letter = PHPExcel_Cell::stringFromColumnIndex($total_cols-1);
		}
		
		return $last_column_letter;
	}
	
	private function set_cell_value_by_column_and_row($sheet, $col, $row, $cval) {
		
		if ($this->can_use_phpspreadsheet) {
			$index = $col + 1;
		} else {
			$index = $col;
		}
        if (strpos($cval, 'FORMAT:::') !== false) {
            $cval = explode(':::', $cval);
            $cell = $sheet->setCellValueByColumnAndRow($index, $row, $cval[2], true);
            $cell->getStyle()->getNumberFormat()->setFormatCode($cval[1]);
        } else {
            $cell = $sheet->setCellValueByColumnAndRow($index, $row, html_entity_decode($cval));
        }
	}

    private function set_cell_value($sheet, $cell, $cval) {
        if (strpos($cval, 'FORMAT:::') !== false) {
            $cval = explode(':::', $cval);
            $cell = $sheet->setCellValue($cell, $cval[2], true);
            $cell->getStyle()->getNumberFormat()->setFormatCode($cval[1]);
        } else {
            $sheet->setCellValue($cell, $cval);
        }
	}
	
	
	function apply_borders_to_cells($sheet, $cell_range, $border_color, $border_position) {
		if ($this->can_use_phpspreadsheet) {
			
			$border_style = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN;
			if ($border_position == 'top_bottom') {
				$border_style_array = array('borders' => array(
						'bottom' => array('borderStyle' => $border_style, 'color' => array('rgb' => $border_color)),
						'top' => array('borderStyle' => $border_style, 'color' => array('rgb' => $border_color)),
				));
			} else {
				$border_style_array = array('borders' => array(
						$border_position => array('borderStyle' => $border_style, 'color' => array('rgb' => $border_color)),
				));
			}
			
		} else {
			
			$border_style = PHPExcel_Style_Border::BORDER_THIN;
			if ($border_position == 'top_bottom') {
				$border_style_array = array('borders' => array(
						'bottom' => array('style' => $border_style, 'color' => array('rgb' => $border_color)),
						'top' => array('style' => $border_style, 'color' => array('rgb' => $border_color)),
				));
			} else {
				$border_style_array = array('borders' => array(
						$border_position => array('style' => $border_style, 'color' => array('rgb' => $border_color))
				));
			}
		}
		
		$sheet->getStyle($cell_range)->applyFromArray($border_style_array);
		
	}

	// -------------------------------------

	function generate_excel_file($sheet_datas = array(), $options = array(), Report $report=null) {
		$logged_user_name = logged_user() instanceof Contact ? logged_user()->getObjectName() : '';
		$properties = array_var($options, 'properties', array());
		$keywords = array_var($properties, 'keywords', "");

		$e = $this->create_spreadsheet_object();
		
		$e->getProperties()
			->setCreator($logged_user_name)
			->setTitle(isset($sheet_datas[0]) && isset($sheet_datas[0]['options']) ? array_var($sheet_datas[0]['options'],'title',"Book 1") : "Book 1")
			->setLastModifiedBy($logged_user_name)
			->setKeywords('excel feng office fengoffice '.$keywords);
		
		$sheet_count = 0;
		foreach ($sheet_datas as $sh_data) {
			if ($sheet_count > 0) {
				$e->createSheet($sheet_count);
			}
			$sheet = $e->getSheet($sheet_count);
			
			$this->generate_excel_file_sheet($sheet, array_var($sh_data, 'headers'), array_var($sh_data, 'data'), array_var($sh_data, 'options'), array_var($sh_data, 'grouped_data'), $report);
			$sheet_count++;
		}
		$writer = $this->get_spreadsheet_writer($e);
		$filename = array_var($options, 'filename', 'excel_'.gen_id().'xlsx');
		$filepath = ROOT.'/tmp/'.$filename;
		$writer->save($filepath);

        return $filepath;
	}
	

	private function generate_excel_file_sheet($sheet, $headers = array(), $data = array(), $sheet_options = array(), $grouped_data = array(), Report $report=null) {
		
		$title = array_var($sheet_options, 'title');
        $conditons = array_var($sheet_options, 'conditions');
		if (strlen_utf($title) > 30) {
			$title = substr_utf($title, 0, 30);
		}
		$title = str_replace(array("*", ":", "/", "\\", "?", "[", "]"), "", $title);
		
		if ($title) $sheet->setTitle($title);
        $data_start_row = 1;

       	/* @var $sheet PhpOffice\PhpSpreadsheet\Worksheet\Worksheet */

        // report conditions
        if (is_array($conditons) && count($conditons) > 0) {
            $total_cols=0;
            foreach ($conditons as $cval) {
            	$this->set_cell_value_by_column_and_row($sheet, $total_cols, $data_start_row, $cval);
                $sheet->mergeCells('A1:E1');
                $total_cols++;
            }

            $data_start_row ++;

        }
        
        $data_groups = array();
        if (!empty($headers) || !empty($data)) {
        	$data_groups[] = array('headers' => $headers, 'data' => $data);
        }
        if (!empty($grouped_data)) {
        	$data_groups = array_merge($data_groups, $grouped_data);
        }

		// GENERATE HEADER BEFORE EVERYTHINHG
		$total_rows = $this->generate_header($sheet, $sheet_options);
		$data_start_row += $total_rows;

        foreach ($data_groups as $data_group) {
        	
        	$headers = array_var($data_group, 'headers', array());
			$data = array_var($data_group, 'data');
			$totals = array_var($data_group, 'totals');
			$totals_after_data = array_var($data_group, 'totals_after_data');
        	
        	$total_cols = 0;
        	
        	// report main headers
        	if (isset($data_group['main_header']) && count($data_group['main_header']) > 0) {
        		$total_cols=0;
        		foreach ($data_group['main_header'] as $hval) {
        			$this->set_cell_value_by_column_and_row($sheet, $total_cols, $data_start_row, $hval);
        			$total_cols++;
        		}
        		
        		$last_column_letter = $this->get_last_column_letter_by_index($total_cols);
        		$main_header = 'A'.$data_start_row.':'.$last_column_letter.$data_start_row;
        		
        		$data_start_row++;
        		
        		$main_header_options = array();
        		$main_header_options['bg_color'] = array_var($sheet_options, 'main_header_bg_color', 'ffffff');
        		$main_header_options['border_color'] = array_var($sheet_options, 'main_header_border_color', 'ffffff');
        		$main_header_options['border_position'] = array_var($sheet_options, 'main_header_border_position', 'bottom');
        		
        		$this->format_table_header($sheet, $main_header, $main_header_options);
        	}
        	
			// report headers
			if (is_array($headers) && count($headers) > 0) {
				$total_cols=0;

				foreach ($headers as $hval) {
					$hval = str_replace(",", "\n", $hval); // when one header is composed by several, separate them to allow text wrap to format the header
					$this->set_cell_value_by_column_and_row($sheet, $total_cols, $data_start_row, $hval);
					$total_cols++;
					if($hval == 'Description' || $hval == 'Projects'){
						// $currentColumn = $this->excelColumnName($total_cols);
						$sheet_options[$total_cols] = 40;
					} elseif ($hval == 'Site Address') {
						// $sheet->getActiveSheet()->getStyle()->getAlignment()->setWrapText(true);

					}elseif($hval == 'Site Address'){
						$sheet_options[$total_cols] = 50;
					}
				}
				
				$last_column_letter = $this->get_last_column_letter_by_index($total_cols);
	
				$header = 'A'.$data_start_row.':'.$last_column_letter.$data_start_row;
	
				$data_start_row++;
	
				$header_options = array();
				if (isset($sheet_options['header_bg_color'])) $header_options['bg_color'] = $sheet_options['header_bg_color'];
				if (isset($sheet_options['header_border_color'])) $header_options['border_color'] = $sheet_options['header_border_color'];
				if (isset($sheet_options['header_border_position'])) $header_options['border_position'] = $sheet_options['header_border_position'];
				$header_options['font_color'] = 'ffffff';
				
				$this->format_table_header($sheet, $header, $header_options);
			}

			// report totals, if defined to be printed before the data
			if (!$totals_after_data && is_array($totals) && count($totals) > 0) {
				$total_cols=0;
				foreach ($totals as $hval) {

					//$hval = $this->removeCurrencies($hval);
					$this->set_cell_value_by_column_and_row($sheet, $total_cols, $data_start_row, $hval);
					$total_cols++;
				}
	
				$last_column_letter = $this->get_last_column_letter_by_index($total_cols);
				$total = 'A'.$data_start_row.':'.$last_column_letter.$data_start_row;
	
				$data_start_row++;
	
				$total_options = array();
				$total_options['bg_color'] = array_var($sheet_options, 'total_bg_color', 'e7e7e7');
				$total_options['border_color'] = array_var($sheet_options, 'total_border_color');
				$total_options['border_position'] = array_var($sheet_options, 'total_border_position', 'top');
				
				$this->format_table_header($sheet, $total, $total_options);
			}
			
			if (is_array($data)) {
				$levelCount = 0;
				$init_row = $data_start_row;
				$alignTop = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP : PHPExcel_Style_Alignment::VERTICAL_TOP;
				foreach ($data as &$row) {
					if (isset($row['type']) && $row['type'] == 'group_header_0' || isset($row['type']) && $row['type'] == 'group_total_0') {
						$spaces = '';
						if ($levelCount == 0) $levelCount++;
					} elseif (isset($row['type']) && $row['type'] == 'group_header_1' || isset($row['type']) && $row['type'] == 'group_total_1') {
						$spaces = '   ';
						if ($levelCount < 2) $levelCount++;
					} elseif (isset($row['type']) && $row['type'] == 'group_header_2' || isset($row['type']) && $row['type'] == 'group_total_2') {
						$spaces = '      ';
						if ($levelCount < 3) $levelCount++;
					} else {
						if ($report != null && $report instanceof Report) {
							if (!$report->getColumnValue("hide_group_details")) {
								$spaces = $this->addSpaces($levelCount);
							} else {
								$spaces = '';
							}
						} else {
							$spaces = '';
						}

					}

					$range_row = 'A' . $init_row . ':' . $last_column_letter . $init_row;

					$bg_color_level_header = 'ffffff';
					$font_color_level_header = '2E4551';
					$font_bold_level_header = false;

					if(isset($row['type'])){
						switch ($row['type']) {
							case 'group_header_2':
								$bg_color_level_header = 'E0E0DF';
								$font_bold_level_header = !$font_bold_level_header;
								break;
							case 'group_header_1':
								$bg_color_level_header = '2E4551';
								$font_color_level_header = 'ffffff';
								break;
							case 'group_total_2':
								$bg_color_level_header = 'E0E0DF';
								break;
							case 'group_total_1':
								$bg_color_level_header = 'E0E0DF';
								$font_bold_level_header = !$font_bold_level_header;
								break;
							case 'group_total_0':
								$bg_color_level_header = 'C5C5C1';
								$font_bold_level_header = !$font_bold_level_header;
								break;
							case 'group_header_0':
								$bg_color_level_header = '2A9D8F';
								$font_color_level_header = 'ffffff';
								$font_bold_level_header = !$font_bold_level_header;
								$border_style = array(
									'borders' => [
										'bottom' => [ 
											'borderStyle' => $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUM : PHPExcel_Style_Border::BORDER_MEDIUM,
											'color' => array('rgb' => '2A9D8F') 
										],
									]
								);
								$sheet->getStyle($range_row)->applyFromArray($border_style);
								break;
		
							default:
								break;
						}
					}else{
						if (($init_row % 2) == 0) $bg_color_level_header = 'F5F8F9';
						$font_alignment = array( 'alignment' => [ 'wrapText' => true, ], );
						$sheet->getStyle($range_row)->applyFromArray($font_alignment);
					}
					
					$this->apply_bg_color_to_cells($sheet, $range_row, $bg_color_level_header);
					$font_style = array('font' => array('bold' => $font_bold_level_header, 'color' => array('rgb' => $font_color_level_header)));
					$sheet->getStyle($range_row)->applyFromArray($font_style);
					$sheet->getStyle($range_row)->getAlignment()->setWrapText(true)->setVertical($alignTop);
					$sheet->getStyle($range_row)->getAlignment()->setVertical($alignTop);
					// $sheet->getRowDimension($init_row)->setRowHeight(-1);
					// $sheet->getRowDimension($init_row)->setRowHeight(13 * (substr_count($sheet->getCell($currentColumn . $init_row)->getValue(), "\r\n") + 2));

					$row = array_diff($row, array("group_header_0", 'group_header_1', 'group_header_2', 'group_total_0', 'group_total_1', 'group_total_2'));
					$row = str_replace(array("\r\n","\r","\n"), " ", $row);

					$total_cols = 0;
					

					foreach ($row as $key => &$rowitem) {
						
						//$rowitem = $this->removeCurrencies($rowitem);

						$decoded = html_entity_decode($rowitem);
						if ($decoded != $rowitem) {
							$decoded = utf8_encode($decoded);
						}

						//$decoded = trim($decoded);
						$rowitem = ($key == 0) ? $spaces.$decoded : $decoded;
						$total_cols++;
					}

					$init_row++;


				}

				// put data after title
                $this->from_array($sheet, $data, ' ', 'A'.$data_start_row);

				// apply borders to data table
				$table_range = 'A'.$data_start_row.':'.$last_column_letter.($init_row-1);
				$this->apply_borders_to_cells($sheet, $table_range, array_var($sheet_options, 'total_border_color', '2E4551'), 'right');
				$this->apply_borders_to_cells($sheet, $table_range, array_var($sheet_options, 'total_border_color', '2E4551'), 'left');
				$this->apply_borders_to_cells($sheet, $table_range, array_var($sheet_options, 'total_border_color', '2E4551'), 'bottom');
				
				$data_start_row += count($data);
			}

			// report totals, if defined to be printed after the data
			if ($totals_after_data && is_array($totals) && count($totals) > 0) {
				$total_cols=0;
				foreach ($totals as $hval) {
					//$hval = $this->removeCurrencies($hval);
					$this->set_cell_value_by_column_and_row($sheet, $total_cols, $data_start_row, $hval);
					$total_cols++;
				}
	
				$last_column_letter = $this->get_last_column_letter_by_index($total_cols);
				$total = 'A'.$data_start_row.':'.$last_column_letter.$data_start_row;
	
				$data_start_row++;
	
				$total_options = array();
				$total_options['bg_color'] = array_var($sheet_options, 'total_bg_color', 'e7e7e7');
				$total_options['border_color'] = array_var($sheet_options, 'total_border_color');
				$total_options['border_position'] = array_var($sheet_options, 'total_border_position', 'bottom');
				
				$this->format_table_header($sheet, $total, $total_options);
			}
        }
        
		// set column autowidth
		try {
			$cellIterator = $sheet->getRowIterator()->current()->getCellIterator();
			//$cellIterator->setIterateOnlyExistingCells(true);
			
			$fixed_cells_width = array_var($sheet_options, 'fixed_cells_width');
			$fixed_description_width = array_var($sheet_options, 'fixed_description_width');

			$numberValue = 1;
			$FirstColumnWidth = 30;

			if (!$fixed_cells_width) {
				foreach ($cellIterator as $cell) {
					if(array_var($sheet_options, $numberValue) > 0){
						$sheet->getColumnDimension($cell->getColumn())->setWidth(array_var($sheet_options, $numberValue));
					}elseif($cell->getColumn() == 'A'){
						$sheet->getColumnDimension($cell->getColumn())->setWidth($FirstColumnWidth);
					}
					else{
						$sheet->getColumnDimension($cell->getColumn())->setAutoSize(true);
					}

					$numberValue++;
				}
			}else {
				foreach ($cellIterator as $cell) {
					$sheet->getColumnDimension($cell->getColumn())->setWidth($fixed_cells_width);
				}
			}
		} catch (Exception $e) {
			Logger::log("Error exporting to excel when setting auto width: ".$e->getMessage());
			Logger::log($e->getMessage());
		}
	}

	private function generate_header($sheet, $sheet_options){
		$data_start_row = 0;
		$center_alingment = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER : PHPExcel_Style_Alignment::HORIZONTAL_CENTER;
		$right_alingment = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT : PHPExcel_Style_Alignment::HORIZONTAL_RIGHT;
		$left_alingment = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT : PHPExcel_Style_Alignment::HORIZONTAL_LEFT;
		$verticalCenterAlingment = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER : PHPExcel_Style_Alignment::VERTICAL_CENTER;
		$bottom_alingment = $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_BOTTOM : PHPExcel_Style_Alignment::VERTICAL_BOTTOM;

		$params = array();
		$params_str = array_var($_REQUEST, 'report_params');
		if ($params_str) $params = json_decode(str_replace("'", '"',$params_str), true);

		$report_id = get_id();
		$filters = build_report_conditions_html($report_id, $params);

		$filters  = str_replace(['<ul>','<li>','</ul>'], ' ',$filters);
		$filters  = str_replace(['</li>'], "\n", $filters);

		$filters = rtrim($filters, "\n");
		
		$company_name = owner_company()->getObjectName();
		$company_logo = null;
		if (FileRepository::isInRepository(owner_company()->getPictureFile())) {
			$company_logo = FileRepository::getBackend()->getFilePath(owner_company()->getPictureFile());
		} else {
			$company_logo = get_product_logo_url(); // default feng logo
		}
		$now_exel = format_datetime(DateTimeValueLib::now());
		$name_report = $sheet_options['title'];
		$name_project = "";
		
		$context = active_context();
		foreach ($context as $selection) :
			if ($selection instanceof Member) :
				$name_project = $selection->getName();
			endif;
		endforeach;

		if ($this->can_use_phpspreadsheet) {
			$drawing = new \PhpOffice\PhpSpreadsheet\Worksheet\MemoryDrawing();
			$drawing->setRenderingFunction(PhpOffice\PhpSpreadsheet\Worksheet\MemoryDrawing::RENDERING_PNG);
			$drawing->setMimeType(PhpOffice\PhpSpreadsheet\Worksheet\MemoryDrawing::MIMETYPE_PNG);
		} else {
			$drawing = new PHPExcel_Worksheet_MemoryDrawing();
			$drawing->setRenderingFunction(PHPExcel_Worksheet_MemoryDrawing::RENDERING_PNG);
			$drawing->setMimeType(PHPExcel_Worksheet_MemoryDrawing::MIMETYPE_PNG);
		}
		$drawing->setName('Company Logo');
		$drawing->setDescription('Company Logo');
		if ($company_logo) {			
			// get image contents and size
			$data = file_get_contents($company_logo);
			$size = getimagesize($company_logo);
			/**
			 * Check if size exists, otherwise,
			 * The image creating function fails.
			 */
			if($size) {
				// create image resource
				$vImg = imagecreatefromstring($data);
				$dstImg = imagecreatetruecolor($size[0], $size[1]);
				// save transaparency
				imagealphablending($dstImg, FALSE);
				imagesavealpha($dstImg, TRUE);
				// generate gd image with transparency from original
				imagecopyresampled($dstImg, $vImg, 0, 0, 0, 0, $size[0], $size[1], $size[0], $size[1]);
				// set the image resource to the Drawing object
				$drawing->setImageResource($dstImg);
			}
		}
		$drawing->setCoordinates('A1');
		$drawing->setOffsetX(10);
		$drawing->setHeight(50);
		$drawing->setWorksheet($sheet);

		$font_style_head = [
			'font' => [ 'size' => 18, 'bold' => true, 'color' => array('rgb' => '2E4760') ],
			'alignment' => ['horizontal' => $center_alingment, 'vertical' => $verticalCenterAlingment]
		];
		$font_style_executed = [
			'font' => [ 'size' => 11, 'bold' => true, 'color' => array('rgb' => 'B1B1B1') ],
			'alignment' => ['horizontal' => $right_alingment, 'vertical' => $bottom_alingment]
		];

		$row = $sheet->getHighestRow()+1;
		$sheet->insertNewRowBefore($row);
		$this->set_cell_value($sheet, 'A1', '');
		$sheet->mergeCells('B1:D1');
		$this->set_cell_value($sheet, 'B1', $company_name);
		$this->set_cell_value($sheet, 'E1', 'Executed on');
		$sheet->getStyle("B1")->applyFromArray($font_style_head);
		$sheet->getStyle("E1")->applyFromArray($font_style_executed);
		$sheet->getRowDimension('1')->setRowHeight(40);
		
		$data_start_row++;

		$font_style_executed_date = [
			'font' => [ 'size' => 10, 'bold' => false, 'color' => array('rgb' => 'B1B1B1') ],
			'alignment' => ['horizontal' => $right_alingment],
		];
		$row = $sheet->getHighestRow()+1;
		$sheet->insertNewRowBefore($row);
		$this->set_cell_value($sheet, 'E2', $now_exel);
		$sheet->getStyle("E2")->applyFromArray($font_style_executed_date);
		$sheet->insertNewRowBefore($row);
		$data_start_row++;

		$font_style_project_title = [
			'font' => [ 'size' => 16, 'bold'  =>  true, 'color' => array('rgb' => '2A9D8F') ],
			'alignment' => ['horizontal' => $center_alingment]
		];
		$font_style_filter_title = [
			'font' => [ 'size' => 12, 'bold'  =>  true, 'color' => array('rgb' => '435762') ],
			'alignment' => ['horizontal' => $right_alingment],
			'borders' => [
				'right' =>
				[
					'borderStyle' => $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUM : PHPExcel_Style_Border::BORDER_MEDIUM,
					'color' => array('rgb' => '2A9D8F')
				],
			]
		];
		
		$row = $sheet->getHighestRow()+2;
		$sheet->insertNewRowBefore($row);
		$sheet->mergeCells('B2:D2');
		$this->set_cell_value($sheet, 'B2', $name_report);
		// $sheet->mergeCells('A4:E4');
		if ($filters) {
			$this->set_cell_value($sheet, 'E3', 'Filters');
		}
		$sheet->getStyle("B2")->applyFromArray($font_style_project_title);
		$sheet->getStyle("E3")->applyFromArray($font_style_filter_title);
		$sheet->getRowDimension('2')->setRowHeight(20);
		$data_start_row++;

		$font_style_project_sbtitle = [
			'font' => [ 'size' => 14, 'bold'  =>  true, 'color' => array('rgb' => '304551') ]
		];
		$font_style_filter_text = [
			'font' => [ 'size' => 11, 'bold' => false, 'color' => array('rgb' => '435762') ],
			'alignment' => ['horizontal' => $right_alingment, 'vertical' => $verticalCenterAlingment],
			'borders' => [
				'right' =>
				[
					'borderStyle' => $this->can_use_phpspreadsheet ? \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUM : PHPExcel_Style_Border::BORDER_MEDIUM,
					'color' => array('rgb' => '2A9D8F')
				],
			]
		];

		$row = $sheet->getHighestRow()+2;
		$sheet->insertNewRowBefore($row);
		$this->set_cell_value($sheet, 'A4', $name_project);
		// $sheet->mergeCells('A5:E5');
		$this->set_cell_value($sheet, 'E4', $filters);
		$sheet->getStyle("A4")->applyFromArray($font_style_project_sbtitle);
		$sheet->getStyle("E4")->applyFromArray($font_style_filter_text);
		$sheet->getRowDimension(4)->setRowHeight(
			12 * (substr_count($sheet->getCell('E4')->getValue(), "\n") + 2)
		);
		
		$data_start_row++;

		$row = $sheet->getHighestRow() + 1;
		$sheet->insertNewRowBefore($row);
		$data_start_row++;

		$report = Reports::instance()->findById($report_id);
		Hook::fire("excel_export_additional_header_content", array('sheet' => $sheet, 'report' => $report), $data_start_row);

		return $data_start_row;
	}
	
	
	private function apply_bg_color_to_cells($sheet, $cell_range, $bg_color) {
		if ($this->can_use_phpspreadsheet) {
			$fill_solid = PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID;
		} else {
			$fill_solid = PHPExcel_Style_Fill::FILL_SOLID;
		}
		
		$sheet->getStyle($cell_range)->getFill()->setFillType($fill_solid)->getStartColor()->setRGB($bg_color);
	}
	
	
	private function format_table_header($sheet, $cell_range, $options = array()) {

		$bg_color = array_var($options, 'bg_color', '333333');
		$border_color = array_var($options, 'border_color', '333333');
		$border_position = array_var($options, 'border_position', 'top_bottom');
		
		if (!in_array($border_position, array('top','bottom','top_bottom'))) $border_position = 'bottom';
		
		// title background color
		$this->apply_bg_color_to_cells($sheet, $cell_range, $bg_color);

		// font style
		if (array_var($options, 'font_color')) :
			// $font_color = array_var($options, 'font_color', '435762');
			$font_style = array('font' => array('bold' => true, 'color' => array('rgb' => array_var($options, 'font_color'))));
		else :
			$font_style = array('font' => array('bold' => true, 'color' => array('rgb' => '435762')));
		endif;
		$sheet->getStyle($cell_range)->applyFromArray($font_style);
		
		// wrap text
		$sheet->getStyle($cell_range)->getAlignment()->setWrapText(true);
		
		// vertical alignment
		if ($this->can_use_phpspreadsheet) {
			$sheet->getStyle($cell_range)->getAlignment()->setVertical(PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER);
		} else {
			$sheet->getStyle($cell_range)->getAlignment()->setVertical(PHPExcel_Style_Alignment::VERTICAL_CENTER);
		}
		
		// borders
		$this->apply_borders_to_cells($sheet, $cell_range, $border_color, $border_position);
		$this->apply_borders_to_cells($sheet, $cell_range, $border_color, 'right');
		$this->apply_borders_to_cells($sheet, $cell_range, $border_color, 'left');
	}
    public function from_array($sheet, array $source = null, $nullValue = null, $startCell = 'A1', $strictNullComparison = false)
    {
        
        //    Convert a 1-D array to 2-D (for ease of looping)
        if (!is_array(end($source))) {
            $source = array($source);
        }

        // start coordinate
        try {
            list ($startColumn, $startRow) = $this->coordinateFromString($startCell);
        } catch (Exception $e) {
            Logger::log("Invalid start cell coordinate: ".$e->getMessage());
            list($startColumn, $startRow) = ['A', 1];
        }

        // Loop through $source
        foreach ($source as $rowData) {
            $currentColumn = $startColumn;
            foreach ($rowData as $cellValue) {
                if ($strictNullComparison) {
                    if ($cellValue !== $nullValue) {
                        $sheet->getCell($currentColumn . $startRow)->setValue($cellValue);
                    }
                } else {
                    if ($cellValue != $nullValue) {
                        // Set cell value
                        $cell = $sheet->getCell($currentColumn . $startRow);
                        if (strpos($cellValue, 'FORMAT:::') !== false) {
                            $cval = explode(':::', $cellValue);
                            $value = $cval[2];
                            $cell->getStyle()->getNumberFormat()->setFormatCode($cval[1]);
                        } else {
                            $value = $cellValue;
                        }
                        $cell->setValue($value);
                    }
                }
                ++$currentColumn;
            }
            ++$startRow;
        }

        return $this;
    }
     /**
     *    Coordinate from string
     *
     *    @param    string    $pCoordinateString
     *    @return    array    Array containing column and row (indexes 0 and 1)
     *    @throws    Exception
     */
    public function coordinateFromString($pCoordinateString = 'A1')
    {
        if (preg_match("/^([$]?[A-Z]{1,3})([$]?\d{1,7})$/", $pCoordinateString, $matches)) {
            return array($matches[1],$matches[2]);
        } elseif ((strpos($pCoordinateString, ':') !== false) || (strpos($pCoordinateString, ',') !== false)) {
            throw new Exception('Cell coordinate string can not be a range of cells');
        } elseif ($pCoordinateString == '') {
            throw new Exception('Cell coordinate can not be zero-length string');
        }

        throw new Exception('Invalid cell coordinate '.$pCoordinateString);
    }

} // ExcelExportController
