<?php
	class Formula extends BaseFormula {

		private $factors_groups = null;
		
		function setFactorsGroups($factors_groups) {
			$this->factors_groups = $factors_groups;
		}
		
		function getFactorsGroups() {
			if ($this->factors_groups == null) {
				$this->factors_groups = $this->loadFactorsGroups();
			}
			return $this->factors_groups;
		}
		
		
		function loadFactorsGroups($order = null) {
			$parameters = array('conditions' => array('formula_id = ?', $this->getId()));
			if ($order) {
				$parameters['order'] = $order;
			}else{
				$parameters['order'] = "status_id DESC";				
			}
			$factors_groups = FormulaFactorsGroups::instance()->findAll($parameters);
			$this->factors_groups = $factors_groups;
		}
		
		
		function deleteAllFactorsGroups() {
			FormulaFactorsGroups::instance()->delete('formula_id = ' . $this->getId());
			$this->factors_groups = null;
		}
		
		
		function calculateStatus(Member $m) {
			$project_ot = ObjectTypes::findByName('project');
			$this->loadFactorsGroups();
			$last_status_id = null;
			$factors_groups_by_status = array();
			
			foreach ($this->factors_groups as $fac) {
				if (!isset($factors_groups_by_status[$fac->getStatusId()])) $factors_groups_by_status[$fac->getStatusId()] = array();
				$fac->loadFactors();
				$factors_groups_by_status[$fac->getStatusId()][] = $fac;
			}
						
			$tasks = ProjectTasks::instance()->listing(array(
					"fire_additional_data_hook" => false,
					'member_ids' => array($m->getId())
			))->objects;
			
			if(count($tasks) == 0){
				return $last_status_id;
			};
			
			$timeslots = Timeslots::instance()->listing(array(
					"fire_additional_data_hook" => false,
					'member_ids' => array($m->getId())
			))->objects;
			
			$status_dimension = Dimensions::findByCode("project_statuses");
			$use_project_status_dimension = $this->getObjectTypeId() == $project_ot->getId() && $status_dimension instanceof Dimension 
				&& in_array($status_dimension->getId(), config_option('enabled_dimensions'));
			
			$ordered_status_ids = array();
			if ($use_project_status_dimension) {
				$ordered_status_ids = $status_dimension->getAllMembers(true, "`order` desc");
			} else {
				foreach ($factors_groups_by_status as $status_id => $factors_groups) {
					$ordered_status_ids[] = $status_id;
				}
			}
			
			//foreach ($factors_groups_by_status as $status_id => $factors_groups) {
			foreach ($ordered_status_ids as $status_id) {
				if (!isset($factors_groups_by_status[$status_id])) continue;
				
				$factors_groups = $factors_groups_by_status[$status_id];
					
				$check_completeness = true;
				$check_dueness = true;
				$check_pending = true;
				$check_estimation = true;

				//all groups of factors for this status
				//OR
				foreach ($factors_groups as $factors_group) {
					$group_factors = $factors_group->getFactors();
				
					//AND
					foreach ($group_factors as $sf) {						
						switch ($sf->getName()) {
							case 'completeness':
								$check_completeness = $sf->getEnabled() ? $this->checkCompleteness($sf, $tasks) : true;
								break;
							case 'dueness':
								$check_dueness = $sf->getEnabled() ? $this->checkDueness($sf, $tasks) : true;
								break;
							case 'pending':
								$check_pending = $sf->getEnabled() ? $this->checkPending($sf, $tasks) : true;
								break;
							case 'estimation':
								$check_estimation = $sf->getEnabled() ? $this->checkEstimation($sf, $tasks, $timeslots) : true;
								break;
						}						
					}	
								
					if ($check_completeness && $check_dueness && $check_pending && $check_estimation) {
						return $status_id;
					}
					
				}
			}
			return null;
		}
		
		
		private function checkCompleteness($factor, $tasks) {
			if(count($tasks) > 0){
				$count_completed = 0;
				foreach ($tasks as $t) {
					if ($t->getCompletedById() > 0) $count_completed++;
				}	
				
				$completed_percent = $count_completed * 100 / count($tasks);
			}else{
				$completed_percent = 0;
			}
			
			return $this->compare_factor_value($factor, $completed_percent);
		}
		
		private function checkDueness($factor, $tasks) {
			$count_due = 0;
			$now_ts = DateTimeValueLib::now()->getTimestamp();
			foreach ($tasks as $t) {
				if ($t->getCompletedById() == 0 && $t->getDueDate() instanceof DateTimeValue && $t->getDueDate()->getTimestamp() < $now_ts) {
					$count_due++;
				}
			}
			
			$due_percent = $count_due * 100 / count($tasks);
			return $this->compare_factor_value($factor, $due_percent);
		}
		
		private function checkPending($factor, $tasks) {
			$count_completed = 0;
			foreach ($tasks as $t) {
				if ($t->getCompletedById() > 0) $count_completed++;
			}
			$count_pending = count($tasks) - $count_completed;
			
			$pending_percent = $count_pending * 100 / count($tasks);
			
			return $this->compare_factor_value($factor, $pending_percent);
		}
		
		private function checkEstimation($factor, $tasks, $timeslots) {
			$minutes_estimated = 0;
			foreach ($tasks as $t) {
				if ($t->getTimeEstimate() > 0) $minutes_estimated += $t->getTimeEstimate();
			}
			
			$minutes_worked = 0;
			foreach ($timeslots as $ts) {
				if ($ts->getMinutes() > 0) $minutes_worked += $ts->getMinutes();
			}
			
			if ($minutes_worked >= $minutes_estimated) {
				$percent_worked = 100;
			} else {
				$percent_worked = $minutes_worked * 100 / $minutes_estimated;
			}
			
			return $this->compare_factor_value($factor, $percent_worked);
		}
		
		
		private function compare_factor_value($factor, $calculated_value) {
			switch ($factor->getComparissonType()) {
				case 'more_than': return $calculated_value > $factor->getRequiredValue();
				case 'more_than_equal_to': return $calculated_value >= $factor->getRequiredValue();
				case 'equal_to': return $calculated_value == $factor->getRequiredValue();
				case 'less_than_equal_to': return $calculated_value <= $factor->getRequiredValue();
				case 'less_than': return $calculated_value < $factor->getRequiredValue();
				default: return false;
			}
		}
		
		
	}	