<?php
/**
 * Quickbooks controller
 *
 * @version 1.0
 * @author Alex Zonis <alex.zonis@fengoffice.com>
 */
class MicrosoftController extends ApplicationController {

	var $plugin_name = "microsoft_integrations";

	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Include Microsoft Graph SDK to use the API
		Env::useLibrary('microsoft_graph', $this->plugin_name);
		Env::useHelper('microsoft_functions', $this->plugin_name);
		Env::useHelper('microsoft_functions_mail', $this->plugin_name);
		
		// only to ensure that the correct config file is put under the config folder;
		get_microsoft_config();
		get_microsoft_mail_config();
	}
	
	/**
	 * Updates subscription to receive event change notifications, if subscription to the calendar does not exists it creates one
	 */
	function update_webhooks() {
		ajx_current("empty"); 
		$params = array_var($_REQUEST, 'parameters');
		$is_mail = array_var($params, 'isMail', false);
		/* Logger::log_r("update_webhooks params: ");
		Logger::log_r($params); */
		try { 
			
			if($is_mail){
				$mail_account_id = array_var($params, 'account_id');
				$account = MailAccounts::instance()->getAccountById($mail_account_id);
				$accessToken = null;
				if ($account instanceof MailAccount) {
					$serialized_token = $account->getColumnValue('oauth2_access_token');
					$accessToken = unserialize(base64_decode($serialized_token));
				}
				$user_id = $mail_account_id;
			}else{
				$accessToken = get_ms_access_token(logged_user()->getId());
				$user_id = logged_user()->getId();
			}
			if (isset($accessToken) && is_valid_microsoft_access_token($user_id, $accessToken, $is_mail)) {
				$graph = new Microsoft\Graph\Graph();
				$graph->setAccessToken($accessToken->getToken());
				
				$event_subscription_present = false;
				$now = DateTimeValueLib::now();
				
				$all_subs_result = $graph->createRequest("GET", "/subscriptions")->execute();
				$all_res_body = $all_subs_result->getBody();
				foreach ($all_res_body['value'] as $sub_array) {
				
				if ($sub_array['resource'] == '/me/events') {
					$event_subscription_present = true;
				}
				
				$expiration = strtotime($sub_array['expirationDateTime']);
				if ($now->getTimestamp() + 600 > $expiration) {
					
					$updated_sub = $this->refresh_webhook($sub_array);
					Logger::log_r("MS UPDATED WEBHOOK: ".print_r($updated_sub,1));
				} else {
					Logger::log_r("MS WEBHOOK STILL VALID: ".print_r($sub_array,1));
				}
				}
				
				if (!$event_subscription_present) {
				$new_sub = $this->create_webhook('/me/events');
				Logger::log_r("MS CREATED WEBHOOK: ".print_r($new_sub,1));
				}
			} else {
				Logger::log("Access Token not valid or not set");
			}
		} catch (Exception $e) {
			Logger::log_r($e->getMessage());
			Logger::log_r($e->getTraceAsString());
		}
	}
	
	/**
	 * Calls the api to refresh a subscription for 2 more days
	 */
	private function refresh_webhook($sub_array) {
		$sub = new Microsoft\Graph\Model\Subscription();
		//$sub->setChangeType("created,updated,deleted");
		//$sub->setNotificationUrl(ROOT_URL."/microsoft_webhook_callback.php");
		//$sub->setResource($sub_array['resource']);
		$time = new \DateTime();
		$time->add(new DateInterval("P2D")); // add two days interval
		$sub->setExpirationDateTime($time);
		
		//$sub = array('expirationDateTime' => $time);
		
		$accessToken = get_ms_access_token(logged_user()->getId());
		$graph = new Microsoft\Graph\Graph();
		$graph->setAccessToken($accessToken->getToken());
		
		$result = $graph->createRequest("PATCH", "/subscriptions/".$sub_array['id'])
		->attachBody($sub)
		->setReturnType(Microsoft\Graph\Model\Subscription::class)
		->execute();
		
		return $result;
	}
	
	/**
	 * Calls the api to create a subscription for 2 days
	 * @param string $resource The object to which the subscription will be associated (/me/events for calendar subscriptions)
	 */
	private function create_webhook($resource) {
		ajx_current("empty");
		
		try {
			$sub = new Microsoft\Graph\Model\Subscription();
			$sub->setChangeType("created,updated,deleted");
			$sub->setNotificationUrl(ROOT_URL."/microsoft_webhook_callback.php");
			$sub->setResource("/me/events");
			$time = new \DateTime();
			$time->add(new DateInterval("P2D"));
			$sub->setExpirationDateTime($time);
			
			$accessToken = get_ms_access_token(logged_user()->getId());
			$graph = new Microsoft\Graph\Graph();
			$graph->setAccessToken($accessToken->getToken());
			
			$subResult = $graph->createRequest("POST", "/subscriptions")
			->attachBody($sub)
			->setReturnType(Microsoft\Graph\Model\Subscription::class)
			->execute();
			
			return $subResult;
			
		} catch (Exception $e) {
			Logger::log_r($e->getMessage());
			Logger::log_r($e->getTraceAsString());
		}
	}
	
	/**
	 * Deletes all the subscriptions created for a calendar
	 */
	function delete_all_webhooks() {
		ajx_current("empty");
		
		$accessToken = get_ms_access_token(logged_user()->getId());
		$graph = new Microsoft\Graph\Graph();
		$graph->setAccessToken($accessToken->getToken());
		
		$all_subs_result = $graph->createRequest("GET", "/subscriptions")->execute();
		$all_res_body = $all_subs_result->getBody();
		Logger::log_r($all_res_body);
		foreach ($all_res_body['value'] as $sub_array) {
			$graph->createRequest("DELETE", "/subscriptions/".$sub_array['id'])->execute();
		
			Logger::log_r("MS SUBSCRIPTION DELETED: ".$sub_array['id']);
		}
	}
	

	function connect_to_microsoft() {
		
		ajx_current("empty");
		
		$action_id = array_var($_REQUEST, 'id');
		$action = array_var($_REQUEST, 'action');
		$action_controller = array_var($_REQUEST, 'controller', 'microsoft');
		
		$accessToken = get_ms_access_token(logged_user()->getId());
		
		if (!isset($accessToken) || !is_valid_microsoft_access_token(logged_user()->getId(), $accessToken)) {
		
			$login_event_data = array(
				'need_microsoft_login' => true,
				'controller' => $action_controller,
				'action' => $action,//'list_user_sub_categories',
				'id' => $action_id,
				'auth_url' => get_microsoft_auth_url(),
			);
			
			$tmp_user_id_file = ROOT."/tmp/user_id_to_connect_to_ms";
			file_put_contents($tmp_user_id_file, logged_user()->getId());
					
			evt_add("login_to_microsoft", $login_event_data);
			
		} else {
			ajx_current("reload");
		}
		
	}
	

	function connect_to_microsoft_mail_account() {
		
		ajx_current("empty");
		
		$action_id = array_var($_REQUEST, 'id');
		$action = array_var($_REQUEST, 'action');
		$action_controller = array_var($_REQUEST, 'controller', 'microsoft');

		$account_id = get_id('account_id');
		
		$account = MailAccounts::instance()->getAccountById($account_id);
		if ($account instanceof MailAccount) {
			$serialized_token = $account->getColumnValue('oauth2_access_token');
			$accessToken = unserialize(base64_decode($serialized_token));
		}
		
		if (!isset($accessToken) || !is_valid_microsoft_access_token(logged_user()->getId(), $accessToken)) {
		
			$login_event_data = array(
				'need_microsoft_login' => true,
				'controller' => $action_controller,
				'action' => $action,
				'id' => $action_id,
				'account_id' => $account_id,
				'auth_url' => get_microsoft_mail_auth_url(),
			);
			
			$tmp_user_id_file = ROOT."/tmp/user_id_to_connect_to_ms";
			file_put_contents($tmp_user_id_file, $account_id);
					
			evt_add("login_to_microsoft_mail", $login_event_data);
			
		} else {
			ajx_current("reload");
		}
		
	}

	function disconnect_from_microsoft_mail_account() {
		$account_id = array_var($_REQUEST, 'account_id');
		delete_microsoft_mail_access_token($account_id);
		ajx_current("reload");
	}

	function download_emails($account = null, $maxPerAccount = null) {
		ajx_current("empty");
		$mailsReceived = 0;
		
		if (!$account instanceof MailAccount) {
			$account_id = array_var($_REQUEST, 'account_id');
			$account = MailAccounts::instance()->getAccountById($account_id);
		}
		
		if ($account instanceof MailAccount) {
			
			$serialized_token = $account->getColumnValue('oauth2_access_token');
			$accessToken = unserialize(base64_decode($serialized_token));

			if (isset($accessToken) && is_valid_microsoft_access_token(logged_user()->getId(), $accessToken)) {

				$graph = new Microsoft\Graph\Graph();
				$graph->setAccessToken($accessToken->getToken());
	
				try {
	
					// create the request to get the emails
					//$response = $graph->createRequest('GET', '/me/todo/lists')->execute();
					//$response = $graph->createRequest('GET', '/me')->execute();
	
					$response = $graph->createRequest('GET', '/me/messages')->execute();

					/***********************************************************
					 * 
					 *	HERE WE NEED TO PROCESS THE RECEIVED EMAILS TO SAVE THEM 
					 *	IN THE DATABASE
					 * 
					 * *********************************************************/
	
				} catch (GuzzleHttp\Exception\ClientException $e) {
	
	
					ajx_current("empty");
					flash_error($e->getMessage());
					Logger::log_r($e->getMessage());
	
					return;
				}
			}

			

			Logger::log_r($response);

			ajx_current("reload");
		}

		return $mailsReceived;
		
	}

	function get_microsoft_imap_emails($account = null, $maxPerAccount = null) {
		ajx_current("empty");
		$mailsReceived = 0;
		
		if (!$account instanceof MailAccount) {
			$account_id = array_var($_REQUEST, 'account_id');
			$account = MailAccounts::instance()->getAccountById($account_id);
		}
		
		if ($account instanceof MailAccount) {
			
			$serialized_token = $account->getColumnValue('oauth2_access_token');
			$accessToken = unserialize(base64_decode($serialized_token));

			if (isset($accessToken) && is_valid_microsoft_access_token(logged_user()->getId(), $accessToken, true)) {


				// or use an array of options instead
				$cm = new ClientManager();

				/** @var \Webklex\PHPIMAP\Client $client */
				$client = $cm->account('mail_account_'.$account_id);

				$access_token = $accessToken->getToken(); 
				$account_email = $account->getEmailAddress();

				// or create a new instance manually
				$client = $cm->make([
					'host'          => 'outlook.office365.com',
					'port'          => 993,
					'encryption'    => 'ssl',
					'validate_cert' => false,
					'username'      => $account_email,
					'password'      => $access_token,
					'protocol'      => 'imap',
					'authentication' => 'oauth'
				]);


				//Connect to the IMAP Server
				$client->connect();

				//Get all Mailboxes
				/** @var \Webklex\PHPIMAP\Support\FolderCollection $folders */
				$folders = $client->getFolders();

				$respons = array();
				//Loop through every Mailbox
				/** @var \Webklex\PHPIMAP\Folder $folder */
				foreach($folders as $folder){

					//Get all Messages of the current Mailbox $folder
					/** @var \Webklex\PHPIMAP\Support\MessageCollection $messages */
					$messages = $folder->messages()->all()->get();
					$response[$folder->getName()] = $messages;
					/** @var \Webklex\PHPIMAP\Message $message */
					/*foreach($messages as $message){
						echo $message->getSubject().'<br />';
						echo 'Attachments: '.$message->getAttachments()->count().'<br />';
						echo $message->getHTMLBody();

						//Move the current Message to 'INBOX.read'
						if($message->move('INBOX.read') == true){
							echo 'Message has ben moved';
						}else{
							echo 'Message could not be moved';
						}
					}*/
				}

				
	
				/* try {
	



	
				} catch (GuzzleHttp\Exception\ClientException $e) {
	
	
					ajx_current("empty");
					flash_error($e->getMessage());
					Logger::log_r($e->getMessage());
	
					return;
				} */
			}

			

			Logger::log_r($response);

			ajx_current("reload");
		}

		return $mailsReceived;
		
	}

	function get_mail_account_info($account = null) {
		ajx_current("empty");
		
		if (!$account instanceof MailAccount) {
			$account_id = array_var($_REQUEST, 'account_id');
			$account = MailAccounts::instance()->getAccountById($account_id);
		}
		
		if ($account instanceof MailAccount) {
			
			$serialized_token = $account->getColumnValue('oauth2_access_token');
			$accessToken = unserialize(base64_decode($serialized_token));
			$account_id = $account->getId();

			if (isset($accessToken) && is_valid_microsoft_access_token($account_id, $accessToken, true)) {

				$graph = new Microsoft\Graph\Graph();
				$graph->setAccessToken($accessToken->getToken());
	
				try {
	
					$response = $graph->createRequest('GET', '/me')->execute();
					
					$response_body = $response->getBody();

					return $response_body;
	
				} catch (GuzzleHttp\Exception\ClientException $e) {
	
	
					ajx_current("empty");
					flash_error($e->getMessage());
					Logger::log_r($e->getMessage());
	
					return;
				}
			}
		}
	}

	function get_mail_account_oauth_access_token($account = null) {
		ajx_current("empty");
		
		if (!$account instanceof MailAccount) {
			$account_id = array_var($_REQUEST, 'account_id');
			$account = MailAccounts::instance()->getAccountById($account_id);
		}
		
		if ($account instanceof MailAccount) {
			
			$serialized_token = $account->getColumnValue('oauth2_access_token');
			$accessToken = unserialize(base64_decode($serialized_token));
			$account_id = $account->getId();

			if (isset($accessToken) && is_valid_microsoft_access_token($account_id, $accessToken, true)) {
				try {
					$token = $accessToken->getToken();
					// Logger::log_r($token);
					return $token;
				} catch (GuzzleHttp\Exception\ClientException $e) {
					ajx_current("empty");
					flash_error($e->getMessage());
					Logger::log_r($e->getMessage());
					return;
				}
			}
		}
	}
	
	function reload_current_panel() {
		ajx_current("reload");
	}
	
	
	function disconnect() {
		delete_ms_access_token(logged_user()->getId());
		ajx_current("reload");
	}
	
	
	function save_event_in_ms_calendar() {
		ajx_current("empty");
		
		$event_id = array_var($_REQUEST, 'id');
		$event = ProjectEvents::instance()->findById($event_id);
		
		if ($event instanceof ProjectEvent) {
			save_feng_event_in_ms_calendar($event, logged_user()->getId());
			ajx_current("reload");
		}
		
	}
	
	
	function delete_event_in_ms_calendar() {
		ajx_current("empty");
		
		$ms_event_id = array_var($_REQUEST, 'id');
		
		if ($ms_event_id) {
			delete_feng_event_in_ms_calendar(logged_user()->getId(), $ms_event_id);
		}
	}

	
}
