<?php 
	class WorkflowPermissions extends BaseWorkflowPermissions {
	
		
		static function checkObjectPermissionsBeforeClassifying($object, $members) {
			
			$ts_members = $object->getMembers();
			$time_status_dim = Dimensions::findByCode('status_timesheet');
			if (!$time_status_dim instanceof Dimension) return true;
			
			foreach ($ts_members as $m) {
				if ($m->getDimensionId() == $time_status_dim->getId()) {
					// if user can't edit the timeslot depemding on the current classification, then this function throws an exception
					WorkflowPermissions::checkIfUserCanEdit(logged_user(), $m, $object);
				}
			}
			
			foreach ($members as $m) {
				if ($m->getDimensionId() == $time_status_dim->getId()) {
					// if user can't classify the timeslot depemding on the next status member, then this function throws an exception
					WorkflowPermissions::checkIfUserCanChangeStatus(logged_user(), $m, $object);
				}
			}
			
		}
		
		
		static function getAllWorkflowPermissionsDefinitionGroupedByObjectType() {
			
			$workflow_permissions = self::instance()->findAll(array(
					"conditions" => "member_id > 0",
					"order" => "object_type_id",
			));
			
			$grouped_permissions = array();
			
			foreach ($workflow_permissions as $workflow_permission) {
				$ot = ObjectTypes::instance()->findById($workflow_permission->getObjectTypeId());
				$ot_name = $ot->getObjectTypeName(true);
				
				if (!isset($grouped_permissions[$ot_name])) $grouped_permissions[$ot_name] = array();
				$grouped_permissions[$ot_name][] = $workflow_permission;
			}
			ksort($grouped_permissions);
			
			return $grouped_permissions;
		}
		
		
		
		static function checkIfUserCanEdit(Contact $user, $member, $object, $throw_exception = true) {
			if ($user->isAdministrator()) return true;

			// if the status change is triggered by an indirect action then don't check permissions 
			// because is the system the one making the change
			if (isset($object->override_workflow_permissions) && $object->override_workflow_permissions) {
				return true;
			}

			$has_permission = true;
			
			$workflow_permission = WorkflowPermissions::getWorkflowPermissionForMember($object->getObjectTypeId(), $member->getId(), 'edit');
			
			if ($workflow_permission) {
				
				$affected_user = null;
				$affected_user_name = "";
				if ($object instanceof Timeslot) {
					$affected_user = $object->getUser();
					$affected_user_name = $affected_user->getName();
				} else if ($object instanceof PaymentReceipt) {
					$affected_user = Contacts::instance()->findById($object->getPaidById());
					$affected_user_name = $affected_user ? $affected_user->getName() : '';
				}
				// check if user can classify this object depending on the timeslot's user and the new classification in status dim
				$has_permission = WorkflowPermissions::userHasWorkflowPermission($user, $workflow_permission, $affected_user);
				if (!$has_permission && $throw_exception) {
					throw new Exception(lang("you dont have permissions to action ot for user", strtolower($workflow_permission->getActionName()), strtolower(ObjectTypes::getPluralObjectTypeName($object->getObjectTypeId())), $affected_user_name));
				}
			}
			
			return $has_permission;
		}
		
		
		
		static function checkIfUserCanChangeStatus($user, $member, $object, $throw_exception = true) {
			if ($user->isAdministrator()) return true;

			// if the status change is triggered by an indirect action then don't check permissions 
			// because is the system the one making the change
			if (isset($object->override_workflow_permissions) && $object->override_workflow_permissions) {
				return true;
			}
			
			$has_permission = true;
			
			$workflow_permission = WorkflowPermissions::getClassificationWorkflowPermissionForMember($object->getObjectTypeId(), $member->getId());
			
			if ($workflow_permission) {
				
				$affected_user = null;
				$affected_user_name = "";
				if ($object instanceof Timeslot) {
					$affected_user = $object->getUser();
					$affected_user_name = $affected_user->getName();
				} else if ($object instanceof PaymentReceipt) {
					$affected_user = Contacts::instance()->findById($object->getPaidById());
					$affected_user_name = $affected_user ? $affected_user->getName() : '';
				}
				// check if user can classify this object depending on the timeslot's user and the new classification in status dim
				$has_permission = WorkflowPermissions::userHasWorkflowPermission($user, $workflow_permission, $affected_user);
				if (!$has_permission && $throw_exception) {
					throw new Exception(lang("you dont have permissions to action ot for user", strtolower($workflow_permission->getActionName()), strtolower(ObjectTypes::getPluralObjectTypeName($object->getObjectTypeId())), $affected_user_name));
				}
			}
			
			return $has_permission;
		}
		
		
		
		static function getWorkflowPermissionForMember($object_type_id, $member_id, $action) {
			return self::instance()->findOne(array("conditions" => array("action=? AND member_id=? AND object_type_id=?", $action, $member_id, $object_type_id)));
		}
		
		
		static function getClassificationWorkflowPermissionForMember($object_type_id, $member_id) {
			return self::instance()->findOne(array("conditions" => array("action != 'edit' AND member_id=? AND object_type_id=?", $member_id, $object_type_id)));
		}
		
		
		static function getWorkflowPermissionByName($permission_name) {
			
			return self::instance()->findOne(array("conditions" => array("`name` = ?", $permission_name)));
			
		}
		
		
		static function userHasWorkflowPermissionByName(Contact $user, $permission_name, $affected_user = null) {
			
			$workflow_permission = WorkflowPermissions::getWorkflowPermissionByName($permission_name);
			
			return self::userHasWorkflowPermissionById($user, $workflow_permission, $affected_user);
			
		}
		
		
		static function userHasWorkflowPermission(Contact $user, WorkflowPermission $workflow_permission, $affected_user = null) {
			
			if (!$workflow_permission) return false;
			
			$user_permission_group_ids = $user->getPermissionGroupIds();
			
			$permission_values = WorkflowPermissionValues::getValuesForPermissionGroups($workflow_permission->getId(), $user_permission_group_ids);
			
			$has_permission = false;
			
			foreach ($permission_values as $permission_value) {
				/* @var $permission_value WorkflowPermissionValue */
				if ($permission_value->getBoolValue()) {
					if ($workflow_permission->getType() == 'boolean_only') {
						$has_permission = true;
						break;
						
					} else if ($workflow_permission->getType() == 'permission_group_list') {
						
						if ($affected_user instanceof Contact) {
							$affected_user_permission_group_ids = $affected_user->getPermissionGroupIds();
							
							$affected_permission_group_ids = WorkflowPermissionValuePgs::getAffectedPermissionGroupIds($permission_value->getId());
							
							$pgs_where_user_has_permissions = array_intersect($affected_permission_group_ids, $affected_user_permission_group_ids);
							
							if (count($pgs_where_user_has_permissions) > 0) {
								$has_permission = true;
								break;
							}
						} else {
							// if object doesn't have user then use the permission's bool value
							// example if permission is for edit, and object doesnt have users then with bool_value we check if the user has the can edit permission
							$has_permission = $permission_value->getBoolValue() > 0;
							break;
						}
					}
				}
			}
			
			return $has_permission;
			
		}
	}
 